/***********************************************
*                                              *
*  C0885 OpenGL Texture Utilities              *
*                                              *
*  Author: Rob Rasile                          *
*                                              *
***********************************************/
#include <windows.h>		// standard Windows app include
#include <fstream>
#include <gl/gl.h>			// standard OpenGL include
#include <gl/glu.h>			// OpenGL utilties
#include <gl/glaux.h>		// OpenGL auxiliary functions
//#include "885TexUtil.h"
#include "EarthTex.h"
//Bitmap(char* filename)
//constructors for Bitmap class
using namespace std;


Bitmap::Bitmap() {}
Bitmap::Bitmap(char* filename)
{
   LoadBitmapFile(filename);
}

// DrawBitmap
// desc: draws the bitmap image data in bitmapData at the location
//		 (rastx, rasty) in the window.  (rastx, rasty) is the lower-left corner
//		 of the bitmap.
void Bitmap::DrawBitmap(long rastx, long rasty)
{
	glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
	glRasterPos2i(rastx, rasty);
	glDrawPixels(bitmapInfoHeader.biWidth, bitmapInfoHeader.biHeight, GL_RGB, GL_UNSIGNED_BYTE, bitmapData);
}

// returns a pointer to the bitmap image data
char* Bitmap::getBitmapData() {
   return bitmapData;
}

// accessor functions for the height and width of the bitmap
long Bitmap::getHeight()
{
   return bitmapInfoHeader.biHeight;
}


long Bitmap::getWidth()
{
   return bitmapInfoHeader.biWidth;
}

// LoadBitmapFile
// desc: Loads the bitmap image of the bitmap specified
//       by filename. Also loads the bitmap header information.
//		 No support for 8-bit bitmaps.
int Bitmap::LoadBitmapFile(char* filename)
{
	ifstream file;								// the file pointer
	int					imageIdx = 0;			// image index counter
	unsigned char		tempRGB;				// swap variable

	file.open(filename, ifstream::binary | ios::in);
	if (!file.is_open())
		return  FAIL;

	// read the bitmap file header
	file.read(reinterpret_cast<char*>(&bitmapFileHeader), sizeof(BITMAPFILEHEADER));
	
	// verify that this is a bitmap by checking for the universal bitmap id
	if (bitmapFileHeader.bfType != BITMAP_ID)
	{
		file.close();
		return  FAIL;
	}

	// read the bitmap information header
	file.read(reinterpret_cast<char*>(&bitmapInfoHeader), sizeof(BITMAPINFOHEADER));

	// move file pointer to beginning of bitmap data
	file.seekg(bitmapFileHeader.bfOffBits, ios::beg);

	// allocate enough memory for the bitmap image data
	bitmapData = new char[bitmapInfoHeader.biSizeImage];

	// verify memory allocation
	if (!bitmapData)
	{
		delete bitmapData;
		file.close();
		return  FAIL;
	}

	// read in the bitmap image data
	file.read(bitmapData, bitmapInfoHeader.biSizeImage);

	// make sure bitmap image data was read
	if (bitmapData == NULL)
	{
		file.close();
		return  FAIL;
	}

	// swap the R and B values to get RGB since the bitmap color format is in BGR
	for (imageIdx = 0; imageIdx < bitmapInfoHeader.biSizeImage; imageIdx+=3)
	{
		tempRGB = bitmapData[imageIdx];
		bitmapData[imageIdx] = bitmapData[imageIdx + 2];
		bitmapData[imageIdx + 2] = tempRGB;
	}

	// close the file and return success
	file.close();
	return SUCCESS;
}

// destructor for Bitmap object... deallocate dynamic data
Bitmap::~Bitmap()
{
   delete bitmapData;
}


// Gluint CreateTestureObject(...) : returns a handle to a texture object. The
// image data is loaded from the file specified by filename.  You must supply the min
// mag filters.  Based on the value of the min filter, mipmapping will be toggled.
// This function supports both 1D and 2D textures.
GLuint CreateTexObject(char* filename, GLenum minfilter, GLenum magfilter, GLenum wrap)
{
    Bitmap bmp(filename);        // bitmap file
    GLenum      type;            // Texture type  (1D or 2D)
    GLuint      texture;         // Texture object (will be returned)
    GLubyte     *ptr;            // Pointer into bit buffer
    GLubyte	*rgba;           // RGBA pixel buffer
    GLubyte	*rgbaptr;        // Pointer into RGBA buffer

    // Figure out the type of texture...
    if (bmp.getHeight() == 1)     // 1D textures have a height of 1
        type = GL_TEXTURE_1D;
    else
        type = GL_TEXTURE_2D;

    // Create and bind a texture object
    glGenTextures(1, &texture);
    glBindTexture(type, texture);

    // Set texture parameters
    glTexParameteri(type, GL_TEXTURE_MAG_FILTER, magfilter);  // magnification filter
    glTexParameteri(type, GL_TEXTURE_MIN_FILTER, minfilter);  // minification filter
    glTexParameteri(type, GL_TEXTURE_WRAP_S, wrap);  // wrap in s direction (s>1)
    glTexParameteri(type, GL_TEXTURE_WRAP_T, wrap);  // wrap in t direction (t>1)

    // Set texture mode - use lighting and state colour for final fragment color calculation
    glTexEnvi(type, GL_TEXTURE_ENV_MODE, GL_MODULATE);


    // Set texture image; if the minification filter uses mip-mapping
    // then use gluBuild2D/1DMipmaps() to load the texture...
    if (minfilter == GL_LINEAR || minfilter == GL_NEAREST)
        glTexImage2D(type, 0, 3, bmp.getWidth(), bmp.getHeight(), 0, GL_RGB,
                              GL_UNSIGNED_BYTE, bmp.getBitmapData());
    else if (type == GL_TEXTURE_1D)
        gluBuild1DMipmaps(type, 3, bmp.getWidth(), GL_RGB,
                              GL_UNSIGNED_BYTE, bmp.getBitmapData());
    else
        gluBuild2DMipmaps(type, 3, bmp.getWidth(), bmp.getHeight(), GL_RGB,
                              GL_UNSIGNED_BYTE, bmp.getBitmapData());

    return (texture);
}



